@@ -1,9 +1,13 @@
 import os
 
 from peewee import (BooleanField, CharField, DateField, ForeignKeyField,
-                    IntegerField, Model, SqliteDatabase)
+                    IntegerField, Model, SqliteDatabase, DoesNotExist)
 from tmc.errors import NoCourseSelected, NoExerciseSelected
 
+# SqliteDatabase will fail if there is no ~/.config
+if not os.path.isdir(os.path.join(os.path.expanduser("~"), ".config")):
+    os.mkdir(os.path.join(os.path.expanduser("~"), ".config"), 0o700)
+
 sqlite = SqliteDatabase(
     os.path.join(os.path.expanduser("~"), ".config", "tmc.db")
 )
@@ -23,8 +27,9 @@ class Course(BaseModel):
     path = CharField(default="")
 
     def set_select(self):
-        Course.update(is_selected=False).where(
-            Course.is_selected == True).execute()
+        Course.update(
+            is_selected=False
+        ).where(Course.is_selected == True).execute()
         self.is_selected = True
         self.save()
 
@@ -66,6 +71,7 @@ class Exercise(BaseModel):
         Exercise.update(is_selected=False).where(
             Exercise.is_selected == True).execute()
         self.is_selected = True
+        self.course.set_select()
         self.save()
 
     def update_downloaded(self):
@@ -112,7 +118,9 @@ class Exercise(BaseModel):
                     realname += " " + c
             else:
                 realname += c
-        return rest.replace("_", " - ") + " - " + realname
+        if len(realname) > 0:
+            return rest.replace("_", " - ") + " - " + realname
+        return self.name
 
 
 class Config(BaseModel):
@@ -121,16 +129,12 @@ class Config(BaseModel):
 
     @staticmethod
     def set(name, value):
-        old = None
         try:
             old = Config.get(Config.name == name)
-        except Config.DoesNotExist:
-            old = None
-        if old is None:
-            Config.create(name=name, value=value)
-        else:
             old.value = value
             old.save()
+        except DoesNotExist:
+            Config.create(name=name, value=value)
 
     @staticmethod
     def get_value(name):
@@ -141,7 +145,7 @@ class Config(BaseModel):
         try:
             Config.get(Config.name == "url")
             return True
-        except Config.DoesNotExist:
+        except DoesNotExist:
             return False
 
 
